@@ -19,6 +19,7 @@ Revision History:
 
 #include "util/warning.h"
 #include "ast/array_decl_plugin.h"
+#include "ast/seq_decl_plugin.h"
 #include "ast/datatype_decl_plugin.h"
 #include "ast/ast_smt2_pp.h"
 #include "ast/ast_pp.h"
@@ -274,7 +275,7 @@ namespace datatype {
                 }
                 parameter const & name = parameters[0];
                 if (!name.is_symbol()) {
-                    TRACE("datatype", tout << "expected symol parameter at position " << 0 << " got: " << name << "\n";);
+                    TRACE("datatype", tout << "expected symbol parameter at position " << 0 << " got: " << name << "\n";);
                     throw invalid_datatype();
                 }
                 for (unsigned i = 1; i < num_parameters; ++i) {
@@ -462,28 +463,27 @@ namespace datatype {
             }
             for (symbol const& s : m_def_block) {
                 def& d = *m_defs[s];
-                for (constructor* c : d) {
-                    for (accessor* a : *c) {
+                for (constructor* c : d) 
+                    for (accessor* a : *c) 
                         a->fix_range(sorts);
-                    }
-                }
             }
-            if (!u().is_well_founded(sorts.size(), sorts.data())) {
+            if (!u().is_well_founded(sorts.size(), sorts.data())) 
                 m_manager->raise_exception("datatype is not well-founded");
-            }
-            if (!u().is_covariant(sorts.size(), sorts.data())) {
+            if (!u().is_covariant(sorts.size(), sorts.data())) 
                 m_manager->raise_exception("datatype is not co-variant");
-            }
-
+ 
             array_util autil(m);
+            seq_util sutil(m);
+            sort* sr;
             for (sort* s : sorts) {
                 for (constructor const* c : get_def(s)) {
                     for (accessor const* a : *c) {
-                        if (autil.is_array(a->range())) {
-                            if (sorts.contains(get_array_range(a->range()))) {
-                                m_has_nested_arrays = true;
-                            }
-                        }                    
+                        if (autil.is_array(a->range()) && sorts.contains(get_array_range(a->range())))
+                            m_has_nested_rec = true;
+                        else if (sutil.is_seq(a->range(), sr) && sorts.contains(sr))
+                            m_has_nested_rec = true;
+                        else if (sutil.is_re(a->range(), sr) && sorts.contains(sr))
+                            m_has_nested_rec = true;
                     }
                 }
             }
@@ -635,6 +635,28 @@ namespace datatype {
             }
         }
 
+        bool plugin::are_distinct(app * a, app * b) const {
+            if (a == b)
+                return false;
+            if (is_unique_value(a) && is_unique_value(b))
+                return true;
+            if (u().is_constructor(a) && u().is_constructor(b)) {
+                if (a->get_decl() != b->get_decl())
+                    return true;
+                for (unsigned i = a->get_num_args(); i-- > 0; ) {
+                    if (!is_app(a->get_arg(i)))
+                        continue;
+                    if (!is_app(b->get_arg(i)))
+                        continue;
+                    app* _a = to_app(a->get_arg(i));
+                    app* _b = to_app(b->get_arg(i));
+                    if (m_manager->are_distinct(_a, _b))
+                        return true;
+                }
+            }
+            return false;
+        }
+
         expr * plugin::get_some_value(sort * s) {
             SASSERT(u().is_datatype(s));
             func_decl * c = u().get_non_rec_constructor(s);
@@ -810,6 +832,10 @@ namespace datatype {
     bool util::is_declared(sort* s) const {
         return plugin().is_declared(s);
     }
+
+    bool util::is_declared(symbol const& n) const {
+        return plugin().is_declared(n);
+    }
     
     void util::compute_datatype_size_functions(svector<symbol> const& names) {
         map<symbol, status, symbol_hash_proc, symbol_eq_proc> already_found;
@@ -882,18 +908,16 @@ namespace datatype {
     bool util::is_well_founded(unsigned num_types, sort* const* sorts) {
         buffer<bool> well_founded(num_types, false);
         obj_map<sort, unsigned> sort2id;
-        for (unsigned i = 0; i < num_types; ++i) {
+        for (unsigned i = 0; i < num_types; ++i) 
             sort2id.insert(sorts[i], i);
-        }
         unsigned num_well_founded = 0, id = 0;
         bool changed;
         ptr_vector<sort> subsorts;
         do {
             changed = false;
             for (unsigned tid = 0; tid < num_types; tid++) {
-                if (well_founded[tid]) {
+                if (well_founded[tid]) 
                     continue;
-                }
                 sort* s = sorts[tid];
                 def const& d = get_def(s);
                 for (constructor const* c : d) {
@@ -901,9 +925,12 @@ namespace datatype {
                         subsorts.reset();
                         get_subsorts(a->range(), subsorts);
                         for (sort* srt : subsorts) {
-                            if (sort2id.find(srt, id) && !well_founded[id]) {
-                                goto next_constructor;
+                            if (sort2id.find(srt, id)) {
+                                if (!well_founded[id]) 
+                                    goto next_constructor;
                             }
+                            else if (is_datatype(srt))
+                                break;
                         }
                     }
                     changed = true;
@@ -1064,11 +1091,9 @@ namespace datatype {
         sort * datatype = con->get_range();
         def const& dd = get_def(datatype);
         symbol r;
-        for (constructor const* c : dd) {
-            if (c->name() == con->get_name()) {
-                r = c->recognizer();
-            }
-        }
+        for (constructor const* c : dd) 
+            if (c->name() == con->get_name()) 
+                r = c->recognizer();                    
         parameter ps[2] = { parameter(con), parameter(r) };
         d  = m.mk_func_decl(fid(), OP_DT_RECOGNISER, 2, ps, 1, &datatype);
         SASSERT(d);
@@ -1103,26 +1128,31 @@ namespace datatype {
         return r;
     }
 
-    bool util::is_recursive_array(sort* a) {
+    bool util::is_recursive_nested(sort* a) {
         array_util autil(m);
-        if (!autil.is_array(a))
-            return false;
-        a = autil.get_array_range_rec(a);                
-        return is_datatype(a) && is_recursive(a);
+        seq_util sutil(m);
+        sort* sr;
+        if (autil.is_array(a)) {
+            a = autil.get_array_range_rec(a);                
+            return is_datatype(a) && is_recursive(a);
+        }
+        if (sutil.is_seq(a, sr))
+            return is_datatype(sr) && is_recursive(sr);
+        if (sutil.is_re(a, sr))
+            return is_datatype(sr) && is_recursive(sr);
+        return false;
     }
 
     bool util::is_enum_sort(sort* s) {
-        if (!is_datatype(s)) {
-            return false;
-        }
+        if (!is_datatype(s)) 
+            return false;        
         bool r = false;
         if (m_is_enum.find(s, r))
             return r;
         ptr_vector<func_decl> const& cnstrs = *get_datatype_constructors(s);
         r = true;
-        for (unsigned i = 0; r && i < cnstrs.size(); ++i) {
-            r = cnstrs[i]->get_arity() == 0;
-        }
+        for (unsigned i = 0; r && i < cnstrs.size(); ++i) 
+            r = cnstrs[i]->get_arity() == 0;        
         m_is_enum.insert(s, r);
         m_asts.push_back(s);
         return r;
@@ -1254,11 +1284,14 @@ namespace datatype {
         unsigned idx = 0;
         def const& d = get_def(f->get_range());
         for (constructor* c : d) {
-            if (c->name() == f->get_name()) {
-                return idx;
-            }
+            if (c->name() == f->get_name()) 
+                return idx;            
             ++idx;
         }
+        IF_VERBOSE(0, verbose_stream() << f->get_name() << "\n");
+        for (constructor* c : d)
+            IF_VERBOSE(0, verbose_stream() << "!= " << c->name() << "\n");
+        SASSERT(false);
         UNREACHABLE();
         return 0;
     }
@@ -1273,14 +1306,22 @@ namespace datatype {
     */
     bool util::are_siblings(sort * s1, sort * s2) {
         array_util autil(m);
-        s1 = autil.get_array_range_rec(s1);                
-        s2 = autil.get_array_range_rec(s2);                
-        if (!is_datatype(s1) || !is_datatype(s2)) {
+        seq_util sutil(m);
+        auto get_nested = [&](sort* s) {
+            while (true) {
+                if (autil.is_array(s))
+                    s = get_array_range(s);
+                else if (!sutil.is_seq(s, s))
+                    break;
+            }
+            return s;
+        };
+        s1 = get_nested(s1);
+        s2 = get_nested(s2);
+        if (!is_datatype(s1) || !is_datatype(s2)) 
             return s1 == s2;
-        }
-        else {
+        else 
             return get_def(s1).id() == get_def(s2).id();
-        }
     }
 
     unsigned util::get_datatype_num_constructors(sort * ty) {
