@@ -114,6 +114,8 @@ app * defined_names::impl::gen_name(expr * e, sort_ref_buffer & var_sorts, buffe
             var_sorts.push_back(s);
         }
         else {
+            domain.push_back(m.mk_bool_sort());
+            new_args.push_back(m.mk_true());
             var_sorts.push_back(m.mk_bool_sort()); // could be any sort.
         }
         var_names.push_back(symbol(i));
@@ -205,7 +207,7 @@ void defined_names::impl::mk_definition(expr * e, app * n, sort_ref_buffer & var
         // the instantiation rules for store(a, i, v) are:
         //     store(a, i, v)[j] = if i = j then v else a[j] with patterns {a[j], store(a, i, v)} { store(a, i, v)[j] }
         // The first pattern is not included.
-        // TBD use a model-based scheme for exracting instantiations instead of
+        // TBD use a model-based scheme for extracting instantiations instead of
         // using multi-patterns.
         // 
 
