@@ -52,7 +52,7 @@ namespace opt {
         if (m_params.m_case_split_strategy == CS_ACTIVITY_DELAY_NEW) {            
             m_params.m_relevancy_lvl = 0;
         }
-        m_params.m_arith_auto_config_simplex = false;
+        m_params.m_arith_auto_config_simplex = true;
         m_params.m_threads = 1; // need to interact with the solver that created model so can't have threads
         // m_params.m_auto_config = false;
     }
@@ -67,7 +67,7 @@ namespace opt {
         m_dump_benchmarks = p.dump_benchmarks();
         m_params.updt_params(_p);
         m_context.updt_params(_p);
-        m_params.m_arith_auto_config_simplex = false;
+        m_params.m_arith_auto_config_simplex = true;
     }
 
     solver* opt_solver::translate(ast_manager& m, params_ref const& p) {
