@@ -369,7 +369,7 @@ namespace sat {
         return result;
     }
 
-    void model_converter::swap(bool_var v, unsigned sz, literal_vector& clause) {
+    void model_converter::swap(bool_var v, unsigned sz, literal_vector& clause) noexcept {
         for (unsigned j = 0; j < sz; ++j) {
             if (v == clause[j].var()) {
                 std::swap(clause[0], clause[j]);
