@@ -115,9 +115,6 @@ public:
     }
 
     solver* translate(ast_manager& dst_m, params_ref const& p) override {
-        if (m_num_scopes > 0) {
-            throw default_exception("Cannot translate sat solver at non-base level");
-        }
         ast_translation tr(m, dst_m);
         m_solver.pop_to_base_level();
         inc_sat_solver* result = alloc(inc_sat_solver, dst_m, p, is_incremental());
