@@ -282,7 +282,7 @@ namespace euf {
     }
 
     void solver::display_validation_failure(std::ostream& out, model& mdl, enode* n) {
-        out << "Failed to validate " << n->bool_var() << " " << bpp(n) << " " << mdl(n->get_expr()) << "\n";
+        out << "Failed to validate b" << n->bool_var() << " " << bpp(n) << " " << mdl(n->get_expr()) << "\n";
         s().display(out);
         euf::enode_vector nodes;
         nodes.push_back(n);
@@ -302,7 +302,7 @@ namespace euf {
             if (mval != sval) {
                 if (r->bool_var() != sat::null_bool_var)
                     out << "b" << r->bool_var() << " ";
-                out << bpp(r) << " :=\neval:  " << sval << "\nmval:  " << mval << "\n";
+                out << bpp(r) << " :=\nvalue obtained from model:  " << sval << "\nvalue of the root expression:  " << mval << "\n";
                 continue;
             }
             if (!m.is_bool(val))
@@ -310,7 +310,7 @@ namespace euf {
             auto bval = s().value(r->bool_var());
             bool tt = l_true == bval;
             if (tt != m.is_true(sval))
-                out << bpp(r) << " :=\neval:  " << sval << "\nmval:  " << bval << "\n";
+                out << bpp(r) << " :=\nvalue according to model:  " << sval << "\nvalue of Boolean literal:  " << bval << "\n";
         }
         for (euf::enode* r : nodes)
             if (r)
@@ -357,6 +357,7 @@ namespace euf {
             if (!tt && !mdl.is_true(e))
                 continue;
             CTRACE("euf", first, display_validation_failure(tout, mdl, n););
+            CTRACE("euf", first, display(tout));
             IF_VERBOSE(0, display_validation_failure(verbose_stream(), mdl, n););
             (void)first;
             first = false;
