@@ -45,7 +45,7 @@ namespace q {
 
         static mam * mk(euf::solver& ctx, ematch& em);
         
-        virtual ~mam() {}
+        virtual ~mam() = default;
         
         virtual void add_pattern(quantifier * q, app * mp) = 0;
 
