@@ -37,7 +37,7 @@ Notes:
 #include "model/model_evaluator.h"
 #include "model/model_v2_pp.h"
 #include "tactic/tactic.h"
-#include "tactic/generic_model_converter.h"
+#include "ast/converters/generic_model_converter.h"
 #include "sat/sat_cut_simplifier.h"
 #include "sat/sat_drat.h"
 #include "sat/tactic/goal2sat.h"
@@ -68,16 +68,14 @@ struct goal2sat::imp : public sat::sat_internalizer {
     sat::solver_core &          m_solver;
     atom2bool_var &             m_map;
     dep2asm_map &               m_dep2asm;
-    obj_map<expr, sat::bool_var>* m_expr2var_replay { nullptr };
+    obj_map<expr, sat::bool_var>* m_expr2var_replay = nullptr;
     bool                        m_ite_extra;
     unsigned long long          m_max_memory;
     expr_ref_vector             m_trail;
     func_decl_ref_vector        m_unhandled_funs;
     bool                        m_default_external;
-    bool                        m_euf { false };
-    bool                        m_drat { false };
-    bool                        m_is_redundant { false };
-    bool                        m_top_level { false };
+    bool                        m_euf = false;
+    bool                        m_top_level = false;
     sat::literal_vector         aig_lits;
     
     imp(ast_manager & _m, params_ref const & p, sat::solver_core & s, atom2bool_var & map, dep2asm_map& dep2asm, bool default_external):
@@ -93,10 +91,6 @@ struct goal2sat::imp : public sat::sat_internalizer {
         updt_params(p);
     }
 
-    ~imp() override {
-    }
-        
-
     sat::cut_simplifier* aig() {
         return m_solver.get_cut_simplifier();
     }
@@ -105,8 +99,7 @@ struct goal2sat::imp : public sat::sat_internalizer {
         sat_params sp(p);
         m_ite_extra  = p.get_bool("ite_extra", true);
         m_max_memory = megabytes_to_bytes(p.get_uint("max_memory", UINT_MAX));
-        m_euf = sp.euf();
-        m_drat = sp.drat_file().is_non_empty_string();
+        m_euf = sp.euf() || sp.smt();
     }
 
     void throw_op_not_handled(std::string const& s) {
@@ -114,8 +107,32 @@ struct goal2sat::imp : public sat::sat_internalizer {
         throw tactic_exception(std::move(s0));
     }
 
-    sat::status mk_status() const {
-        return sat::status::th(m_is_redundant, m.get_basic_family_id());
+    symbol m_tseitin = symbol("tseitin");
+
+    euf::th_proof_hint* mk_tseitin(unsigned n, sat::literal const* lits) {
+        if (m_euf && ensure_euf()->use_drat())
+            return ensure_euf()->mk_smt_hint(m_tseitin, n, lits);
+        return nullptr;
+    }
+
+    euf::th_proof_hint* mk_tseitin(sat::literal a, sat::literal b) {
+        if (m_euf && ensure_euf()->use_drat()) {
+            sat::literal lits[2] = { a, b };
+            return ensure_euf()->mk_smt_hint(m_tseitin, 2, lits);
+        }
+        return nullptr;
+    }
+
+    euf::th_proof_hint* mk_tseitin(sat::literal a, sat::literal b, sat::literal c) {
+        if (m_euf && ensure_euf()->use_drat()) {
+            sat::literal lits[3] = { a, b, c };
+            return ensure_euf()->mk_smt_hint(m_tseitin, 3, lits);
+        }
+        return nullptr;
+    }
+
+    sat::status mk_status(euf::th_proof_hint* ph = nullptr) const {
+        return sat::status::th(false, m.get_basic_family_id(), ph);
     }
 
     bool relevancy_enabled() {
@@ -124,48 +141,44 @@ struct goal2sat::imp : public sat::sat_internalizer {
 
     bool top_level_relevant() {
         return m_top_level && relevancy_enabled();
-    }
-    
-    void mk_clause(sat::literal l) {
-        mk_clause(1, &l);
-    }
+    }   
 
-    void mk_clause(sat::literal l1, sat::literal l2) {
+    void mk_clause(sat::literal l1, sat::literal l2, euf::th_proof_hint* ph) {
         sat::literal lits[2] = { l1, l2 };
-        mk_clause(2, lits);
+        mk_clause(2, lits, ph);
     }
 
-    void mk_clause(sat::literal l1, sat::literal l2, sat::literal l3) {
+    void mk_clause(sat::literal l1, sat::literal l2, sat::literal l3, euf::th_proof_hint* ph) {
         sat::literal lits[3] = { l1, l2, l3 };
-        mk_clause(3, lits);
+        mk_clause(3, lits, ph);
     }
 
-    void mk_clause(unsigned n, sat::literal * lits) {
+    void mk_clause(unsigned n, sat::literal * lits, euf::th_proof_hint* ph) {
         TRACE("goal2sat", tout << "mk_clause: "; for (unsigned i = 0; i < n; i++) tout << lits[i] << " "; tout << "\n";);
         if (relevancy_enabled())
             ensure_euf()->add_aux(n, lits);
-        m_solver.add_clause(n, lits, mk_status());
+        m_solver.add_clause(n, lits, mk_status(ph));
     }
 
     void mk_root_clause(sat::literal l) {
         mk_root_clause(1, &l);
     }
 
-    void mk_root_clause(sat::literal l1, sat::literal l2) {
+    void mk_root_clause(sat::literal l1, sat::literal l2, euf::th_proof_hint* ph = nullptr) {
         sat::literal lits[2] = { l1, l2 };
-        mk_root_clause(2, lits);
+        mk_root_clause(2, lits, ph);
     }
 
-    void mk_root_clause(sat::literal l1, sat::literal l2, sat::literal l3) {
+    void mk_root_clause(sat::literal l1, sat::literal l2, sat::literal l3, euf::th_proof_hint* ph = nullptr) {
         sat::literal lits[3] = { l1, l2, l3 };
-        mk_root_clause(3, lits);
+        mk_root_clause(3, lits, ph);
     }
 
-    void mk_root_clause(unsigned n, sat::literal * lits) {
+    void mk_root_clause(unsigned n, sat::literal * lits, euf::th_proof_hint* ph = nullptr) {
         TRACE("goal2sat", tout << "mk_root_clause: "; for (unsigned i = 0; i < n; i++) tout << lits[i] << " "; tout << "\n";);
         if (relevancy_enabled())
             ensure_euf()->add_root(n, lits);
-        m_solver.add_clause(n, lits, m_is_redundant ? mk_status() : sat::status::input());
+        m_solver.add_clause(n, lits, ph ? mk_status(ph) : sat::status::input());
     }
 
     sat::bool_var add_var(bool is_ext, expr* n) {
@@ -173,15 +186,11 @@ struct goal2sat::imp : public sat::sat_internalizer {
         if (m_expr2var_replay && m_expr2var_replay->find(n, v))
             return v;
         v = m_solver.add_var(is_ext);
-        log_def(v, n);
+        if (!is_ext && m_euf && ensure_euf()->use_drat())
+            ensure_euf()->set_bool_var2expr(v, n);
         return v;
     }
 
-    void log_def(sat::bool_var v, expr* n) {
-        if (m_drat && m_euf)
-            ensure_euf()->drat_bool_def(v, n);
-    }
-
     sat::bool_var to_bool_var(expr* e) override {
         sat::literal l;
         sat::bool_var v = m_map.to_bool_var(e);
@@ -266,6 +275,18 @@ struct goal2sat::imp : public sat::sat_internalizer {
         m_lit2app.insert(l.index(), t);
         m_cache_trail.push_back(t);
     }
+
+    sat::literal get_cached(app* t) const override {
+        sat::literal lit = sat::null_literal;
+        m_app2lit.find(t, lit);
+        return lit;
+    }
+
+    bool is_cached(app* t, sat::literal l) const override {
+        sat::literal lit = get_cached(t);
+        SASSERT(lit == sat::null_literal || l == lit);
+        return l == lit;
+    }
     
     void convert_atom(expr * t, bool root, bool sign) {       
         SASSERT(m.is_bool(t));
@@ -274,11 +295,15 @@ struct goal2sat::imp : public sat::sat_internalizer {
         if (v == sat::null_bool_var) {
             if (m.is_true(t)) {
                 sat::literal tt = sat::literal(mk_bool_var(t), false);
+                if (m_euf && ensure_euf()->use_drat())
+                    ensure_euf()->set_bool_var2expr(tt.var(), t);
                 mk_root_clause(tt);
                 l = sign ? ~tt : tt;
             }
             else if (m.is_false(t)) {
                 sat::literal ff = sat::literal(mk_bool_var(t), false);
+                if (m_euf && ensure_euf()->use_drat())
+                    ensure_euf()->set_bool_var2expr(ff.var(), t);
                 mk_root_clause(~ff);
                 l = sign ? ~ff : ff;
             }
@@ -405,13 +430,15 @@ struct goal2sat::imp : public sat::sat_internalizer {
             m_result_stack.shrink(old_sz);
         }
         else {
+            if (process_cached(t, root, sign))
+                return;
             SASSERT(num <= m_result_stack.size());
             sat::bool_var k = add_var(false, t);
             sat::literal  l(k, false);
             cache(t, l);
             sat::literal * lits = m_result_stack.end() - num;       
             for (unsigned i = 0; i < num; i++) 
-                mk_clause(~lits[i], l);
+                mk_clause(~lits[i], l, mk_tseitin(~lits[i], l));
                        
             m_result_stack.push_back(~l);
             lits = m_result_stack.end() - num - 1;
@@ -421,7 +448,7 @@ struct goal2sat::imp : public sat::sat_internalizer {
             }
             // remark: mk_clause may perform destructive updated to lits.
             // I have to execute it after the binary mk_clause above.
-            mk_clause(num+1, lits);
+            mk_clause(num+1, lits, mk_tseitin(num+1, lits));
             if (aig()) 
                 aig()->add_or(l, num, aig_lits.data());
                         
@@ -454,6 +481,8 @@ struct goal2sat::imp : public sat::sat_internalizer {
             m_result_stack.shrink(old_sz);
         }
         else {
+            if (process_cached(t, root, sign))
+                return;
             SASSERT(num <= m_result_stack.size());
             sat::bool_var k = add_var(false, t);
             sat::literal  l(k, false);
@@ -462,7 +491,7 @@ struct goal2sat::imp : public sat::sat_internalizer {
 
             // l => /\ lits
             for (unsigned i = 0; i < num; i++) {
-                mk_clause(~l, lits[i]);
+                mk_clause(~l, lits[i], mk_tseitin(~l, lits[i]));
             }
             // /\ lits => l
             for (unsigned i = 0; i < num; ++i) {
@@ -474,7 +503,7 @@ struct goal2sat::imp : public sat::sat_internalizer {
                 aig_lits.reset();
                 aig_lits.append(num, lits);
             }
-            mk_clause(num+1, lits);
+            mk_clause(num+1, lits, mk_tseitin(num+1, lits));
             if (aig()) {
                 aig()->add_and(l, num, aig_lits.data());
             }        
@@ -484,7 +513,6 @@ struct goal2sat::imp : public sat::sat_internalizer {
             
             m_result_stack.shrink(old_sz);
             m_result_stack.push_back(l);
-            TRACE("goal2sat", tout << m_result_stack << "\n";);
         }
     }
 
@@ -507,16 +535,18 @@ struct goal2sat::imp : public sat::sat_internalizer {
             }
         }
         else {
+            if (process_cached(n, root, sign))
+                return;
             sat::bool_var k = add_var(false, n);
             sat::literal  l(k, false);
             cache(n, l);
-            mk_clause(~l, ~c, t);
-            mk_clause(~l,  c, e);
-            mk_clause(l,  ~c, ~t);
-            mk_clause(l,   c, ~e);
+            mk_clause(~l, ~c, t, mk_tseitin(~l, ~c, t));
+            mk_clause(~l,  c, e, mk_tseitin(~l, c, e));
+            mk_clause(l,  ~c, ~t, mk_tseitin(l, ~c, ~t));
+            mk_clause(l,   c, ~e, mk_tseitin(l, c, ~e));
             if (m_ite_extra) {
-                mk_clause(~t, ~e, l);
-                mk_clause(t,  e, ~l);
+                mk_clause(~t, ~e, l, mk_tseitin(~t, ~e, l));
+                mk_clause(t,  e, ~l, mk_tseitin(t, e, ~l));
             }
             if (aig()) aig()->add_ite(l, c, t, e);
             if (sign)
@@ -537,12 +567,14 @@ struct goal2sat::imp : public sat::sat_internalizer {
             mk_root_clause(sign ? lit : ~lit);            
         }
         else {
+            if (process_cached(t, root, sign))
+                return;
             sat::bool_var k = add_var(false, t);
             sat::literal  l(k, false);
             cache(t, l);
             // l <=> ~lit
-            mk_clause(lit, l);
-            mk_clause(~lit, ~l);
+            mk_clause(lit, l, mk_tseitin(lit, l));
+            mk_clause(~lit, ~l, mk_tseitin(~lit, ~l));
             if (sign)
                 l.neg();
             m_result_stack.push_back(l);
@@ -567,13 +599,15 @@ struct goal2sat::imp : public sat::sat_internalizer {
             }            
         }
         else {
+            if (process_cached(t, root, sign))
+                return;
             sat::bool_var k = add_var(false, t);
             sat::literal  l(k, false);
             cache(t, l);
             // l <=> (l1 => l2)
-            mk_clause(~l, ~l1, l2);
-            mk_clause(l1, l);
-            mk_clause(~l2, l);
+            mk_clause(~l, ~l1, l2, mk_tseitin(~l, ~l1, l2));
+            mk_clause(l1, l, mk_tseitin(l1, l));
+            mk_clause(~l2, l, mk_tseitin(~l2, l));
             if (sign)
                 l.neg();
             m_result_stack.push_back(l);
@@ -603,14 +637,16 @@ struct goal2sat::imp : public sat::sat_internalizer {
             }                  
         }
         else {
+            if (process_cached(t, root, sign))
+                return;
             sat::bool_var k = add_var(false, t);
             sat::literal  l(k, false);
             if (m.is_xor(t))
                 l1.neg();
-            mk_clause(~l, l1, ~l2);
-            mk_clause(~l, ~l1, l2);
-            mk_clause(l, l1, l2);
-            mk_clause(l, ~l1, ~l2);
+            mk_clause(~l,  l1, ~l2, mk_tseitin(~l, l1, ~l2));
+            mk_clause(~l, ~l1,  l2, mk_tseitin(~l, ~l1, l2));
+            mk_clause(l,   l1,  l2, mk_tseitin(l, l1, l2));
+            mk_clause(l,  ~l1, ~l2, mk_tseitin(l, ~l1, ~l2));
             if (aig()) aig()->add_iff(l, l1, l2);
 
             cache(t, l);
@@ -656,7 +692,7 @@ struct goal2sat::imp : public sat::sat_internalizer {
         sat::literal lit;
         {
             flet<bool> _top(m_top_level, false);
-            lit = euf->internalize(e, sign, root, m_is_redundant);           
+            lit = euf->internalize(e, sign, root);           
         }
         if (lit == sat::null_literal) 
             return;
@@ -679,7 +715,7 @@ struct goal2sat::imp : public sat::sat_internalizer {
             th = dynamic_cast<euf::th_solver*>(ext);
             SASSERT(th);
         }
-        auto lit = th->internalize(t, sign, root, m_is_redundant);
+        auto lit = th->internalize(t, sign, root);
         m_result_stack.shrink(m_result_stack.size() - t->get_num_args());
         if (lit == sat::null_literal)
             return;
@@ -748,12 +784,11 @@ struct goal2sat::imp : public sat::sat_internalizer {
         }
     };
 
-    void process(expr* n, bool is_root, bool redundant) {
+    void process(expr* n, bool is_root) {
         TRACE("goal2sat", tout << "process-begin " << mk_bounded_pp(n, m, 2) 
             << " root: " << is_root 
             << " result-stack: " << m_result_stack.size() 
             << " frame-stack: " << m_frame_stack.size() << "\n";);
-        flet<bool> _is_redundant(m_is_redundant, redundant);
         scoped_stack _sc(*this, is_root);
         unsigned sz = m_frame_stack.size();
         if (visit(n, is_root, false)) 
@@ -804,14 +839,14 @@ struct goal2sat::imp : public sat::sat_internalizer {
             << " result-stack: " << m_result_stack.size() << "\n";);
     }
 
-    sat::literal internalize(expr* n, bool redundant) override {
+    sat::literal internalize(expr* n) override {
         bool is_not = m.is_not(n, n);
         flet<bool> _top(m_top_level, false);
         unsigned sz = m_result_stack.size();
         (void)sz;
         SASSERT(n->get_ref_count() > 0);
         TRACE("goal2sat", tout << "internalize " << mk_bounded_pp(n, m, 2) << "\n";);
-        process(n, false, redundant);
+        process(n, false);
         SASSERT(m_result_stack.size() == sz + 1);
         sat::literal result = m_result_stack.back();
         TRACE("goal2sat", tout << "done internalize " << result << " " << mk_bounded_pp(n, m, 2) << "\n";);
@@ -857,7 +892,7 @@ struct goal2sat::imp : public sat::sat_internalizer {
         flet<bool> _top(m_top_level, true);
         VERIFY(m_result_stack.empty());
         TRACE("goal2sat", tout << "assert: " << mk_bounded_pp(n, m, 3) << "\n";);
-        process(n, true, m_is_redundant);
+        process(n, true);
         CTRACE("goal2sat", !m_result_stack.empty(), tout << m_result_stack << "\n";);
         SASSERT(m_result_stack.empty());
     }
@@ -882,22 +917,44 @@ struct goal2sat::imp : public sat::sat_internalizer {
         m_result_stack.pop_back();
     }
 
+    struct scoped_reset {
+        imp& i;
+        scoped_reset(imp& i) :i(i) {}
+        ~scoped_reset() {
+            i.m_interface_vars.reset();
+            i.m_app2lit.reset();
+            i.m_lit2app.reset();
+        }
+    };
+    
+    void operator()(unsigned n, expr* const* fmls) {
+        scoped_reset _reset(*this);
+        // collect_boolean_interface(g, m_interface_vars);
+        for (unsigned i = 0; i < n; ++i) 
+            process(fmls[i]);
+    }
+
+    void assumptions(unsigned n, expr* const* fmls) {
+        scoped_reset _reset(*this);
+        // collect_boolean_interface(g, m_interface_vars);
+        for (unsigned i = 0; i < n; ++i) {
+            expr* f = fmls[i];
+            expr* f1 = f;
+            bool sign = m.is_not(f, f1);
+            insert_dep(f, f1, sign);
+        }
+    }
+
+
     void operator()(goal const & g) {
-        struct scoped_reset {
-            imp& i;
-            scoped_reset(imp& i) :i(i) {}
-            ~scoped_reset() {
-                i.m_interface_vars.reset();
-                i.m_app2lit.reset();
-                i.m_lit2app.reset();
-            }
-        };
         scoped_reset _reset(*this);
         collect_boolean_interface(g, m_interface_vars);
         unsigned size = g.size();
         expr_ref f(m), d_new(m);
         ptr_vector<expr> deps;
         expr_ref_vector  fmls(m);
+        if (m_euf)
+            ensure_euf();
         for (unsigned idx = 0; idx < size; idx++) {
             f = g.form(idx);
             // Add assumptions.
@@ -935,7 +992,7 @@ struct goal2sat::imp : public sat::sat_internalizer {
     void update_model(model_ref& mdl) {
         auto* ext = dynamic_cast<euf::solver*>(m_solver.get_extension());
         if (ext)
-            ext->update_model(mdl);
+            ext->update_model(mdl, true);
     }
 
     void user_push() {
@@ -985,21 +1042,45 @@ goal2sat::~goal2sat() {
     dealloc(m_imp);
 }
 
+euf::solver* goal2sat::ensure_euf() {
+    return m_imp->ensure_euf();
+}
+
+
 void goal2sat::collect_param_descrs(param_descrs & r) {
     insert_max_memory(r);
     r.insert("ite_extra", CPK_BOOL, "(default: true) add redundant clauses (that improve unit propagation) when encoding if-then-else formulas");
 }
 
-
-void goal2sat::operator()(goal const & g, params_ref const & p, sat::solver_core & t, atom2bool_var & m, dep2asm_map& dep2asm, bool default_external) {
+void goal2sat::init(ast_manager& m, params_ref const & p, sat::solver_core & t, atom2bool_var & map, dep2asm_map& dep2asm, bool default_external) {
     if (!m_imp) {
-        m_imp = alloc(imp, g.m(), p, t, m, dep2asm, default_external);
+        m_imp = alloc(imp, m, p, t, map, dep2asm, default_external);
         for (unsigned i = 0; i < m_scopes; ++i)
             m_imp->user_push();
     }
+}
+
+void goal2sat::operator()(goal const & g, params_ref const & p, sat::solver_core & t, atom2bool_var & m, dep2asm_map& dep2asm, bool default_external) {
+    init(g.m(), p, t, m, dep2asm, default_external);
     (*m_imp)(g);    
 }
 
+void goal2sat::operator()(unsigned n, expr* const* fmls) {
+    SASSERT(m_imp);
+    (*m_imp)(n, fmls);
+}
+
+void goal2sat::assumptions(unsigned n, expr* const* fmls) {
+    SASSERT(m_imp);
+    m_imp->assumptions(n, fmls);
+}
+
+sat::literal goal2sat::internalize(expr* a) {
+    SASSERT(m_imp);
+    return m_imp->internalize(a);
+}
+
+
 void goal2sat::get_interpreted_funs(func_decl_ref_vector& funs) {
     if (m_imp) 
         funs.append(m_imp->interpreted_funs());
