@@ -99,7 +99,7 @@ namespace smt {
         theory_var result = null_theory_var;
         numeral range;
         numeral new_range;
-        numeral small_range_thresold(1024);
+        numeral small_range_threshold(1024);
         unsigned n = 0;
         for (row const& row : m_rows) {
             theory_var v = row.get_base_var();
@@ -117,7 +117,7 @@ namespace smt {
             numeral const & u = upper_bound(v).get_rational();
             new_range  = u;
             new_range -= l;
-            if (new_range > small_range_thresold) {
+            if (new_range > small_range_threshold) {
                 //
             }
             else if (result == null_theory_var || new_range < range) {
@@ -503,7 +503,7 @@ namespace smt {
         theory_var x_i = r.get_base_var();
         
         SASSERT(is_int(x_i));
-        // The following assertion is wrong. It may be violated in mixed-real-interger problems.
+        // The following assertion is wrong. It may be violated in mixed-real-integer problems.
         // The check is_gomory_cut_target will discard rows where any variable contains infinitesimals.
         // SASSERT(m_value[x_i].is_rational()); // infinitesimals are not used for integer variables
         SASSERT(!m_value[x_i].is_int());     // the base variable is not assigned to an integer value.
