@@ -26,6 +26,7 @@ Revision History:
 #include "ast/pattern/pattern_inference.h"
 #include "ast/macros/quasi_macros.h"
 #include "ast/occurs.h"
+#include "ast/bv_decl_plugin.h"
 #include "solver/assertions/asserted_formulas.h"
 
 
@@ -54,6 +55,7 @@ asserted_formulas::asserted_formulas(ast_manager & m, smt_params & sp, params_re
     m_elim_bvs_from_quantifiers(*this),
     m_cheap_quant_fourier_motzkin(*this),
     m_apply_bit2int(*this),
+    m_bv_size_reduce(*this),
     m_lift_ite(*this),
     m_ng_lift_ite(*this),
     m_find_macros(*this),
@@ -154,10 +156,14 @@ void asserted_formulas::assert_expr(expr * e, proof * _in_pr) {
     force_push();
     proof_ref  in_pr(_in_pr, m), pr(_in_pr, m);
     expr_ref   r(e, m);
+    SASSERT(m.is_bool(e));
 
     if (inconsistent())
         return;
 
+    if (m.is_true(e))
+        return;
+
     if (m_smt_params.m_preprocess) {
         TRACE("assert_expr_bug", tout << r << "\n";);
         set_eliminate_and(false); // do not eliminate and before nnf.
@@ -168,7 +174,7 @@ void asserted_formulas::assert_expr(expr * e, proof * _in_pr) {
             else
                 pr = m.mk_modus_ponens(in_pr, pr);
         }
-        TRACE("assert_expr_bug", tout << "after...\n" << r << "\n";);
+        TRACE("assert_expr_bug", tout << "after...\n" << r << "\n" << pr << "\n";);
     }
 
     m_has_quantifiers |= ::has_quantifiers(e);
@@ -193,7 +199,7 @@ void asserted_formulas::push_scope_core() {
     reduce();
     commit();
     SASSERT(inconsistent() || m_qhead == m_formulas.size() || m.limit().is_canceled());
-    TRACE("asserted_formulas_scopes", tout << "before push: " << m_scopes.size() << "\n";);
+    TRACE("asserted_formulas_scopes", tout << "before push: " << m_scopes.size() << "\n");
     m_scoped_substitution.push();
     m_scopes.push_back(scope());
     scope & s = m_scopes.back();
@@ -204,8 +210,9 @@ void asserted_formulas::push_scope_core() {
     m_elim_term_ite.push();
     m_bv_sharing.push_scope();
     m_macro_manager.push_scope();
+    m_bv_size_reduce.push_scope();
     commit();
-    TRACE("asserted_formulas_scopes", tout << "after push: " << m_scopes.size() << "\n";);
+    TRACE("asserted_formulas_scopes", tout << "after push: " << m_scopes.size() << "\n");
 }
 
 void asserted_formulas::force_push() {
@@ -224,6 +231,7 @@ void asserted_formulas::pop_scope(unsigned num_scopes) {
     TRACE("asserted_formulas_scopes", tout << "before pop " << num_scopes << " of " << m_scopes.size() << "\n";);
     m_bv_sharing.pop_scope(num_scopes);
     m_macro_manager.pop_scope(num_scopes);
+    m_bv_size_reduce.pop_scope(num_scopes);
     unsigned new_lvl    = m_scopes.size() - num_scopes;
     scope & s           = m_scopes[new_lvl];
     m_inconsistent      = s.m_inconsistent_old;
@@ -274,6 +282,8 @@ void asserted_formulas::reduce() {
     TRACE("before_reduce", display(tout););
     CASSERT("well_sorted", check_well_sorted());
 
+    IF_VERBOSE(10, verbose_stream() << "(smt.simplify-begin :num-exprs " << get_total_size() << ")\n";);
+
     set_eliminate_and(false); // do not eliminate and before nnf.
     if (!invoke(m_propagate_values)) return;
     if (!invoke(m_find_macros)) return;
@@ -292,6 +302,7 @@ void asserted_formulas::reduce() {
     if (!invoke(m_find_macros)) return;
     if (!invoke(m_apply_quasi_macros)) return;
     if (!invoke(m_apply_bit2int)) return;
+    if (!invoke(m_bv_size_reduce)) return;
     if (!invoke(m_cheap_quant_fourier_motzkin)) return;
     if (!invoke(m_pattern_inference)) return;
     if (!invoke(m_max_bv_sharing_fn)) return;
@@ -300,7 +311,7 @@ void asserted_formulas::reduce() {
     if (!invoke(m_flatten_clauses)) return;
 //    if (!invoke(m_propagate_values)) return;
 
-    IF_VERBOSE(10, verbose_stream() << "(smt.simplifier-done)\n";);
+    IF_VERBOSE(10, verbose_stream() << "(smt.simplifier-done :num-exprs " << get_total_size() << ")\n";);
     TRACE("after_reduce", display(tout););
     TRACE("after_reduce_ll", ast_mark visited; display_ll(tout, visited););
     TRACE("macros", m_macro_manager.display(tout););
@@ -321,13 +332,13 @@ unsigned asserted_formulas::get_formulas_last_level() const {
 
 bool asserted_formulas::invoke(simplify_fmls& s) {
     if (!s.should_apply()) return true;
-    IF_VERBOSE(10, verbose_stream() << "(smt." << s.id() << ")\n";);
     s();
+    IF_VERBOSE(10, verbose_stream() << "(smt." << s.id() << " :num-exprs " << get_total_size() << ")\n";);
     IF_VERBOSE(10000, verbose_stream() << "total size: " << get_total_size() << "\n";);
     TRACE("reduce_step_ll", ast_mark visited; display_ll(tout, visited););
     CASSERT("well_sorted",check_well_sorted());
+    TRACE("after_reduce", display(tout << s.id() << "\n"););
     if (inconsistent() || canceled()) {
-        TRACE("after_reduce", display(tout););
         TRACE("after_reduce_ll", ast_mark visited; display_ll(tout, visited););
         return false;
     }
@@ -499,7 +510,8 @@ void asserted_formulas::simplify_fmls::operator()() {
         else {
             af.push_assertion(result, result_pr, new_fmls);
         }
-        if (af.canceled()) return;
+        if (af.canceled())
+            return;
     }
     af.swap_asserted_formulas(new_fmls);
     TRACE("asserted_formulas", af.display(tout););
@@ -508,9 +520,9 @@ void asserted_formulas::simplify_fmls::operator()() {
 
 
 void asserted_formulas::reduce_and_solve() {
-    IF_VERBOSE(10, verbose_stream() << "(smt.reducing)\n";);
     flush_cache(); // collect garbage
     m_reduce_asserted_formulas();
+    IF_VERBOSE(10, verbose_stream() << "(smt.reduced " << get_total_size() << ")\n";);
 }
 
 
@@ -716,6 +728,40 @@ void asserted_formulas::refine_inj_axiom_fn::simplify(justified_expr const& j, e
 }
 
 
+void asserted_formulas::bv_size_reduce_fn::simplify(justified_expr const& j, expr_ref& n, proof_ref& p) {
+    bv_util bv(m);
+    expr* f = j.get_fml();
+    expr* a, *b, *x;
+    unsigned lo, hi;
+    rational r;
+    expr_ref new_term(m);
+    auto check_reduce = [&](expr* a, expr* b) {
+        if (bv.is_extract(a, lo, hi, x) && lo > 0 && hi + 1 == bv.get_bv_size(x) && bv.is_numeral(b, r) && r == 0) {
+            // insert x -> x[0,lo-1] ++ n into sub
+            new_term = bv.mk_concat(b, bv.mk_extract(lo - 1, 0, x));
+            m_sub.insert(x, new_term);
+            n = j.get_fml();
+            return true;
+        }
+        return false;
+    };
+    if (m.is_eq(f, a, b) && (check_reduce(a, b) || check_reduce(b, a))) {
+        // done
+    }
+    else {
+        n = j.get_fml();
+        m_sub(n);
+    }
+}
+
+void asserted_formulas::bv_size_reduce_fn::push_scope() {
+    m_sub.push_scope();
+}
+
+void asserted_formulas::bv_size_reduce_fn::pop_scope(unsigned n) {
+    m_sub.pop_scope(n);
+}
+
 unsigned asserted_formulas::get_total_size() const {
     expr_mark visited;
     unsigned r  = 0;
@@ -726,6 +772,7 @@ unsigned asserted_formulas::get_total_size() const {
 
 
 #ifdef Z3DEBUG
+#include <iostream>
 void pp(asserted_formulas & f) {
     f.display(std::cout);
 }
